@@ -7,8 +7,10 @@
 #include <glib.h>
 #include <glib-object.h>
 
+G_BEGIN_DECLS
+
 /* Work around bugs in PyGILState api fixed in 2.4.0a4 */
-#if PY_HEXVERSION < 0x020400A4
+#if PY_VERSION_HEX < 0x020400A4
 #define PYGIL_API_IS_BUGGY TRUE
 #else
 #define PYGIL_API_IS_BUGGY FALSE
@@ -64,6 +66,9 @@ typedef struct {
 #define PyGParamSpec_Get(v) (((PyGParamSpec *)v)->pspec)
 #define PyGParamSpec_Check(v) (PyObject_TypeCheck(v, &PyGParamSpec_Type))
 
+typedef int (*PyGClassInitFunc) (gpointer gclass, PyTypeObject *pyclass);
+
+
 struct _PyGObject_Functions {
     void (* register_class)(PyObject *dict, const gchar *class_name,
 			    GType gtype, PyTypeObject *type, PyObject *bases);
@@ -137,14 +142,14 @@ struct _PyGObject_Functions {
 					const GParamSpec* pspec);
     PyTypeObject *enum_type;
     PyObject *(*enum_add)(PyObject *module,
-			  const char *typename,
+			  const char *typename_,
 			  const char *strip_prefix,
 			  GType gtype);
     PyObject* (*enum_from_gtype)(GType gtype, int value);
     
     PyTypeObject *flags_type;
     PyObject *(*flags_add)(PyObject *module,
-			   const char *typename,
+			   const char *typename_,
 			   const char *strip_prefix,
 			   GType gtype);
     PyObject* (*flags_from_gtype)(GType gtype, int value);
@@ -153,6 +158,8 @@ struct _PyGObject_Functions {
     int       (*enable_threads) (void);
     int       (*gil_state_ensure) (void);
     void      (*gil_state_release) (int flag);
+    void      (*register_class_init) (GType gtype, PyGClassInitFunc class_init);
+    void      (*register_interface_info) (GType gtype, const GInterfaceInfo *info);
 };
 
 #ifndef _INSIDE_PYGOBJECT_
@@ -204,6 +211,8 @@ struct _PyGObject_Functions *_PyGObject_API;
 #define pyg_flags_add               (_PyGObject_API->flags_add)
 #define pyg_flags_from_gtype        (_PyGObject_API->flags_from_gtype)
 #define pyg_enable_threads          (_PyGObject_API->enable_threads)
+#define pyg_register_class_init     (_PyGObject_API->register_class_init)
+#define pyg_register_interface_info (_PyGObject_API->register_interface_info)
 
 #define pyg_block_threads()   G_STMT_START {   \
     if (_PyGObject_API->block_threads != NULL) \
@@ -214,23 +223,21 @@ struct _PyGObject_Functions *_PyGObject_API;
       (* _PyGObject_API->unblock_threads)();     \
   } G_STMT_END
 
-#define pyg_threads_enabled (_PyGObject_API->threads_enabled)
 
-#define pyg_gil_state_ensure() (_PyGObject_API->threads_enabled? (_PyGObject_API->gil_state_ensure()) : 0)
+#if PY_VERSION_HEX < 0x020400F0
+# define pyg_threads_enabled (_PyGObject_API->threads_enabled)
+#else
+# define pyg_threads_enabled (PyEval_ThreadsInitialized())
+#endif
+
+#define pyg_gil_state_ensure() (pyg_threads_enabled? (_PyGObject_API->gil_state_ensure()) : 0)
 #define pyg_gil_state_release(state) G_STMT_START {     \
-    if (_PyGObject_API->threads_enabled)                \
+    if (pyg_threads_enabled)                            \
         _PyGObject_API->gil_state_release(state);       \
     } G_STMT_END
 
-#define pyg_begin_allow_threads                 \
-    G_STMT_START {                              \
-        PyThreadState *_save = NULL;            \
-        if (_PyGObject_API->threads_enabled)    \
-            _save = PyEval_SaveThread();
-#define pyg_end_allow_threads                   \
-        if (_PyGObject_API->threads_enabled)    \
-            PyEval_RestoreThread(_save);        \
-    } G_STMT_END
+#define pyg_begin_allow_threads Py_BEGIN_ALLOW_THREADS
+#define pyg_end_allow_threads Py_END_ALLOW_THREADS
 
 #define init_pygobject() { \
     PyObject *gobject = PyImport_ImportModule("gobject"); \
@@ -253,4 +260,6 @@ struct _PyGObject_Functions *_PyGObject_API;
 
 #endif /* !_INSIDE_PYGOBJECT_ */
 
+G_END_DECLS
+
 #endif /* !_PYGOBJECT_H_ */
